#ifndef R_XSL_H
#define R_XSL_H

/*
 Taken from the Extensions example on the X-alan C++ page 
 */

 // Base header file.  Must be first.
#include <Include/PlatformDefinitions.hpp>
#include <util/PlatformUtils.hpp>
#include <XalanTransformer/XalanTransformer.hpp>

#include <cmath>
#include <ctime>
#include <iostream>
#include <fstream>


#include "RSCommon.h"
#undef error

void init_R();


class SSignature {
 protected:
   int numArgs; 
   int next;
   XObject::eObjectType *argTypes;
   char **names;

 public:
   SSignature() { numArgs = 0; argTypes = NULL; names = NULL; next = 0;}
   SSignature(int nargs) {
                                numArgs = nargs;
                                argTypes = (XObject::eObjectType*) calloc(sizeof(XObject::eObjectType), numArgs);
                                names = NULL;
                                next = 0;
                         }

   int add(XObject::eObjectType v) { argTypes[next++] = v; return(next);}
};


class FunctionRCall : public Function
{
public:

  /**
   * Execute an XPath function object.  The function must return a valid
   * XObject.
   *
   * @param executionContext executing context
   * @param context          current context node
   * @param opPos            current op position
   * @param args             vector of pointers to XObject arguments
   * @return                 pointer to the result XObject
   */
  virtual XObjectPtr
  execute(
      XPathExecutionContext& executionContext,
      XalanNode* context,
      int  /* opPos */,
      const XObjectArgVectorType&    args);

  virtual XObjectPtr
  execute(XPathExecutionContext&			executionContext,
  	  XalanNode*						context);


  /**
   * Implement clone() so Xalan can copy the function into
   * its own function table.
   *
   * @return pointer to the new object
   */
// For compilers that do not support covariant return types,
// clone() must be declared to return the base type.
#if defined(XALAN_NO_COVARIANT_RETURN_TYPE)
  virtual Function*
#else
  virtual FunctionRCall*
#endif
  clone() const
  {
    return new FunctionRCall(*this);
  }


  protected:
    SSignature *signature;
    XObject::eObjectType returnType;

  public:
    FunctionRCall() { returnType = XObject::eUnknown; signature = NULL;}

    SSignature *getSignature(){ return(signature);}
    void        setSignature(SSignature *s){ signature = s;}

    void setReturnType(XObject::eObjectType val) {returnType = val;}
    XObject::eObjectType  getReturnType() {return(returnType);}

 protected:
    virtual USER_OBJECT_ function() { return(NULL_USER_OBJECT); }

 public:
    virtual const char *functionName() const { return(NULL); }

    USER_OBJECT_ convert(const XObjectPtr &arg);
    USER_OBJECT_ convert(const XalanNode *node);
    XObjectPtr convert(USER_OBJECT_ val, XPathExecutionContext& executionContext);
    XObjectPtr convertXMLNode(USER_OBJECT_ val, XPathExecutionContext& executionContext);

 private:
    // The assignment and equality operators are not implemented...
  FunctionRCall&  operator=(const FunctionRCall&);
  bool  operator==(const FunctionRCall&) const;
};

class NamedRFunction : public FunctionRCall
{
 protected:
   const char *_functionName;

 public:
   NamedRFunction(const char *funName) { _functionName = strdup(funName); };
   NamedRFunction(const NamedRFunction &other) { _functionName = other.functionName(); };


 const char * functionName() const {
  return(_functionName);
 }

#if defined(XALAN_NO_COVARIANT_RETURN_TYPE)
  virtual Function*
#else
  virtual NamedRFunction*
#endif
  clone() const
  {
    return new NamedRFunction(*this);
  }
};


USER_OBJECT_ fromXSLString(XObjectPtr arg);
bool isXSLInteger(char *tmp, long *lval);
bool isXSLReal(char *tmp, double *dval);

#endif

