#include "SXSLTUtils.hpp"

#include "SXalanException.hpp"

#include "process.hpp"


XObjectPtr *SXSL_getNodeFromRef(USER_OBJECT_ ref);

/*
 We should be able to do this directly from the c_str() method
 but that seems to be returning a string with a single character.
*/
char *
getString(const XalanDOMString &str)
{
  char *val;
  int n = str.length();
   val = (char *)calloc(n+1, sizeof(char));
  for(int i = 0; i < n; i ++)
     val[i] = (char) str[i];

 return(val);
}


static int RInitialized = 0;
extern "C" {
 extern int Rf_initEmbeddedR(int argc, char *argv[]);
}

/**
  The default initialization of R using the built-in command line arguments.
*/
void
init_R()
{
  bool mustFree = false;
  char **argv; 
  char *defaults[] = {"Sxalan", "--silent"};
  int nargs = CommandLineParameters->ScommandArgs.size();
   if(nargs == 0) {
      nargs = 2;
      argv = (char **)malloc(sizeof(char*) * (nargs));
      for(int i = 0 ; i < nargs ; i++) 
        argv[i] = strdup(defaults[i]);
   } else {
     mustFree = false;
     argv = (char **)malloc(sizeof(char*) * (nargs+1));
     argv[0] = strdup("Sxalan");
     for(int i = 0 ; i < nargs ; i++) {
      argv[i+1] = strdup(CommandLineParameters->ScommandArgs[i]);
     }
     nargs++;
   }

  init_R(nargs, argv);

  if(mustFree)
    free(argv);
}

/**
 Initialize the R engine allowing the user to specify
 the command line arguments.
*/
void
init_R(int argc, char *args[])
{
 if(RInitialized)
      return;

  Rf_initEmbeddedR(argc, args);

  USER_OBJECT_ libName = NEW_CHARACTER(1);
   SET_STRING_ELT(libName, 0, COPY_TO_USER_STRING("SXalan"));
  if(S_call1ArgFunction("library", libName) == NULL) {
    fprintf(stderr, "No R library SXalan. Check the value of R_LIBS");
    fflush(stderr);
  }

  RInitialized = 1;
}

/**
 Call an S function with a single argument.
 This is a convenience routine to avoid the caller having to create
 the argument list for a single value.
*/
USER_OBJECT_
S_call1ArgFunction(const char *funcName, USER_OBJECT_ arg)
{
 USER_OBJECT_ e;

 PROTECT(e = allocVector(LANGSXP, 2));

  USER_OBJECT_ fun = Rf_findFun(Rf_install((char *)funcName),  R_GlobalEnv);
  if(fun == NULL || GET_LENGTH(fun) == 0) {
    return(NULL);
  }
  SETCAR(e, fun);
  SETCAR(CDR(e), arg);
  USER_OBJECT_ val = eval(e, R_GlobalEnv);
  UNPROTECT(1);

 return(val);
}



/*
 Checks S3-style class inheritance directly rather than
 via the inherits() function.
*/
int 
R_IS(USER_OBJECT_ obj, const char * const name)
{
 int i, n;
 USER_OBJECT_ className;
 
 className = GET_CLASS(obj);
 n = GET_LENGTH(className);
 
  for(i=0;i<n;i++) {
    if(strcmp(name, CHAR_DEREF(STRING_ELT(className, i))) == 0) {
       return(TRUE);
    }
  }
 return(FALSE);
}


/*
 Handles errors that occur in the embedded R and then
 throws an exception in Xalan.
*/

void
jump_now()
{
  fprintf(stderr, "Error in R\n");  fflush(stderr);

  throw SXalanException("Error in R");
}


extern "C" {
  USER_OBJECT_ SXSL_getNodeName(USER_OBJECT_ ref);
  USER_OBJECT_ SXSL_getNodeAttributes(USER_OBJECT_ ref);
}

USER_OBJECT_ 
SXSL_getNodeName(USER_OBJECT_ ref)
{
  USER_OBJECT_ ans;
  const XObjectPtr *obj = SXSL_getNodeFromRef(ref);
  const NodeRefListBase &nodes = (*obj)->nodeset();

  XalanNode *node = nodes.item(0);   
  const XalanDOMString& name = node->getNodeName();
  char *tmp = getString(name);
  PROTECT(ans = NEW_CHARACTER(1));
  SET_STRING_ELT(ans, 0, COPY_TO_USER_STRING(tmp));
  UNPROTECT(1);
  return(ans);
}


XObjectPtr *
SXSL_getNodeFromRef(USER_OBJECT_ ref)
{
  double tmp = NUMERIC_DATA(VECTOR_ELT(ref, 0))[0];

  return((XObjectPtr *) ((long) tmp));
}
